;;: -*- emacs-lisp -*-
;;; pi-functions.el
;;; Author: Philippe Ivaldi
;;; Last modified: Sun Jul  1 01:02:00 CEST 2007

(eval-when-compile
  (require 'cl))

;; ------------------------------------------------
;; * Permutation alatoire de lettres ds un texte *
;; Slectionner un texte et M-x scramble-region <RET>
;; Auteur: http://christophe.deleuze.free.fr/emacs.html
(defun create-permutation (n)
  "Return as a vector a permutation of the first n integers (including 0)."
  (defun swap (vec i j)
    (let ((vec_i (aref vec i)))
      (aset vec i (aref vec j))
      (aset vec j vec_i)))
  
  (let ((prmt (make-vector n 0)) i)
    (dotimes (i n) (aset prmt i i))
    (dotimes (i n) (swap prmt (random n) (random n)))
    prmt
    ))

;;;###autoload
(defun scramble-word (&optional arg)
  "Randomly permute non-border letters in word at point."
  (interactive "P")
  (let* ((bow (progn (backward-word 1) (point)))
	 (eow (progn (forward-word 1) (point)))
	 (word (buffer-substring (1+ bow) (1- eow)))
	 (len  (length word))
	 (prmt (create-permutation len))
	 (rslt (make-string len ?a))
	 i)
    (if (< len 2) ()
      (dotimes (i len)
	(aset rslt i (aref word (aref prmt i))))
      (goto-char (1+ bow))
      (delete-char len)
      (insert rslt)
      (forward-word 1))
    ))

;;;###autoload
(defun scramble-region (beg end)
  "Randomly permute non-border letters of words in region."
  (interactive "r")
  (save-excursion
    (goto-char beg)
    (while (< (point) end)
      (scramble-word)
      (forward-word 1))))
;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;###autoload
(defun xpdf()
  "* Pour voir le .pdf associ au buffer."
  (interactive)
  (let ((tcom (concat "xpdf -z page "  (file-name-sans-extension (buffer-file-name)) ".pdf &")))
    (if (buffer-modified-p) (save-buffer))
    (shell-command tcom)))

;;;###autoload
(defun acro()
  "* Pour voir le .pdf associ au buffer avec acroread."
  (interactive)
  (let ((tcom (concat "acroread "  (file-name-sans-extension (buffer-file-name)) ".pdf &")))
    (if (buffer-modified-p) (save-buffer))
    (shell-command tcom)))

;;;###autoload
(defun gv()
  "* Pour voir le .pdf associ au buffer avec acroread."
  (interactive)
  (let ((tcom (concat "gv "  (file-name-sans-extension (buffer-file-name)) ".pdf &")))
    (if (buffer-modified-p) (save-buffer))
    (shell-command tcom)))

;;;;;;;;;;;;;;;;;;;;


;; Author: David Biesack (David.Biesack@sas.com  or biesack@mindspring.com)
(defvar scissors "8<------"
  "string to insert in \\[separe]")

;;;###autoload
(defun separe ()
  "Insert a line of SCISSORS in the buffer"
  (interactive)
  (or (bolp) (beginning-of-line 2))
  (while (<= (current-column) (- (or fill-column 70) (length scissors)))
    (insert scissors))
  (newline))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;; -----------------------------------------
;; * Conversion d'expressions parenthses *
;; Par exemple, slectionner la ligne:
;; (a + b * (c + d* (e + f))) - a * (c + d (e + f))
;; M-x big-paren-region <RET> produit:
;; \Big(a + b * \big(c + d* (e + f)\big)\Big) - a * \big(c + d (e + f)\big)
(defun match-paren (arg)
  "Go to the matching parenthesis if on parenthesis."
  ;;  (interactive "p")
  (cond ((looking-at "[([{]") (forward-sexp 1) (backward-char))
	((looking-at "[])}]") (forward-char) (backward-sexp 1))
	(t )))

(defun paren-depth-at-point (begin)
  ;;Retourne la profondeur du point dans les parenthses
  (save-excursion
    (let ((thedepth -1) (notbegin t))
      (if (eq ?\) (char-after (point)))
	  (backward-char 1))
      (while (and (>= (point) begin) notbegin) ;;Problme avec le dbut
	;;du buffer
	(if (eq ?\( (char-after (point)))
	    (incf thedepth)
	  (if (eq ?\) (char-after (point)))
	      (decf thedepth)))
	(if (not (bobp)) (backward-char 1) (setq notbegin nil))
	)
      thedepth)))

(defun paren-max-depth-region (beginr endr)
  ;;Retourne le profondeur maximale des parenthses de la rgion
  (interactive "r")
  (save-excursion
    (let ((max-depth 0) (curr-depth 0))
      (goto-char beginr)
      (while (and (< (point) endr) (re-search-forward "(" nil t 1))
	(setq curr-depth (paren-depth-at-point beginr))
	(if (> curr-depth max-depth)
	    (setq max-depth curr-depth)))
      max-depth)))

(defvar mm-big-liste '("" "\\big" "\\Big" "\\bigg" "\\Bigg"))

(defun match-paren-point ()
  (let ((pos 0))
    (match-paren 1)
    (setq pos (point))
    (match-paren 1)
    pos))

;;;###autoload
(defun big-paren-region (beginr endr)
  "Convertit la rgion avec les \big"
  (interactive "r")
  (let ((list mm-big-liste) (maxdepth 0) (paren-size 0))
    (narrow-to-region beginr endr)
    (goto-char beginr)
    (when (re-search-forward "(*)" nil t 1)
      (setq maxdepth (paren-max-depth-region beginr endr)) ;;Prof max
      (if (< maxdepth 5)
	  (progn
	    (goto-char beginr)
	    (while (re-search-forward "(" nil t 1)
	      (backward-char 1)
	      (setq paren-size (paren-max-depth-region (point) (match-paren-point)))
	      (insert (nth paren-size list))
	      (match-paren 1)
	      (insert (nth paren-size list))
	      (match-paren 1)
	      (forward-char 1)
	      ))
	(message "Profondeur maximale dpasse..."))
      (widen) ;;En cas de pb C-x n w
      )))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;###autoload
(defun pi-replace-regexp (find replace)
  (interactive "sFind\nsRemplace par :")
  (while (re-search-forward find nil t)
    (replace-match replace nil nil)))

;; ----------------------------------------------
;; * Retourne le code pour dfinir un raccourci *
;;;###autoload
(defun moy-insert-set-key (key function)
  "Retourne le code pour dfinir un raccourci.
Auteur: Matthieu MOY."
  (interactive "kKey to bind:
aFunction to bind: ")
  (insert "(global-set-key (kbd \"" (key-description key) "\") '"
          (symbol-name function) ")"))

;; ------------------------
;; * LaTeX vers ascii-art *
;;;###autoload
(defun textomail-region (beginr endr)
  ;;Convertir la rgion de LaTeX en ascii-art
  (interactive "r")
  (write-region beginr endr "~/textomail" nil nil)
  (kill-region beginr endr)
  (insert (shell-command-to-string "perl /home/pi/bin/tex2mail ~/textomail"))
  )

;; ------------------------------------------
;; * Increments any numbers found in a line *
;; From ;;http://www.emacswiki.org/cgi-bin/wiki/IncrementNumber
;;;###autoload
(defun pi-increment-number-line (num-lines)
  "Copies line, preserving cursor column, and increments any numbers found.
  Copies a block of optional NUM-LINES lines.  If no optional argument is given,
  then only one line is copied."
  (interactive "p")
  (if (not num-lines) (setq num-lines 0) (setq num-lines (1- num-lines)))
  (let* ((col (current-column))
         (bol (save-excursion (forward-line (- num-lines)) (beginning-of-line) (point)))
         (eol (progn (end-of-line) (point)))
         (line (buffer-substring bol eol)))
    (goto-char bol)
    (while (re-search-forward "[0-9]+" eol 1)
      (let ((num (string-to-number (buffer-substring
                                    (match-beginning 0) (match-end 0)))))
        (replace-match (int-to-string (1+ num))))
      (setq eol (save-excursion (goto-char eol) (end-of-line) (point))))
    (goto-char bol)
    (insert line "\n")
    (move-to-column col)))

;;;###autoload
(defun pi-increment-number-decimal (&optional arg)
  "Increment the number forward from point by 'arg'."
  (interactive "p*")
  (save-excursion
    (save-match-data
      (let (inc-by field-width answer)
        (setq inc-by (if arg arg 1))
        (skip-chars-backward "0123456789")
        (when (re-search-forward "[0-9]+" nil t)
          (setq field-width (- (match-end 0) (match-beginning 0)))
          (setq answer (+ (string-to-number (match-string 0) 10) inc-by))
          (when (< answer 0)
            (setq answer (+ (expt 10 field-width) answer)))
          (replace-match (format (concat "%0" (int-to-string field-width) "d")
                                 answer)))))))


;; ----------------------------------------
;; * Insertion de code html dans des .asy *
;;  en vue d'tre moulin par asy-verbatin-new.sh
;;;###autoload
(defun pi-asy-url()
  (interactive)
  (insert "/*html
Texte....:
<a href=\"url.tml\">TEXTE</a>
html*/
"))

;; ;;;###autoload
;; (defun pi-asy-doc ()
;;   ""
;;   (interactive)
;;   (insert "/*DOC")
;;   )

;; -------------------------------------------------------------------
;; * Dump current region to a file without changing the visited file *
;; From http://www.emacswiki.org/cgi-bin/wiki/PaulRivier
;;;###autoload
(defun dump-region-to-file (filename)
  "Dump current region to a file without changing the visited file"
  (interactive "fFilename:")
  (save-excursion
    (let ((str (buffer-substring-no-properties (mark) (point))))
      (with-temp-buffer (progn
    			  (insert str)
    			  (write-file filename t))))))


;; -------------------------------
;; * no-xls pour ouvrir des .xsl *
;;;###autoload
(defun no-xls (&optional filename)
  "Run xlhtml and w3m -dump on the entire buffer.
    Optional FILENAME says what filename to use.
    This is only necessary for buffers without
    proper `buffer-file-name'.  FILENAME should
    be a real filename, not a path."
  (interactive "fExcel File: ")
  (when (and filename
             (not (buffer-file-name)))
    (write-file (make-temp-file filename)))
  (erase-buffer)
  (shell-command
   (format "xlhtml -nc -te %s | w3m -dump -T text/html" (buffer-file-name))
   (current-buffer))
  (setq buffer-file-name nil)
  (set-buffer-modified-p nil))
(add-to-list 'auto-mode-alist '("\\.xls\\'" . no-xls))


(defun pi-last-modified()
  (interactive)
  (insert (concat "Last modified: " (shell-command-to-string "LANG=US date"))))
(global-set-key (kbd "<f11>") 'pi-last-modified)
(message "pi-functions.el(c) is loaded.")